#!/usr/bin/python2.7

from requiredpackagesformss import *
import coalescentdatastructures as cds
import microsatelitedatastructures as mds
from excelutilities import *

#print help(cds)

def makeinitcellv1(msnumber,initialMSL):
	"""function that formats a cell class from mds for use to start filling in microsatelite patterns for cells"""
	return mds.Cell(msnumber,True,initialMSL,mds.simpleslippageandpointmutation,"lldp-slp",15,.01,.957, False, False)

def makecoalescentmicrosatelitetree(initnumcells=cds.initnumcells,inverlambda=cds.inverlambda,startingcell=makeinitcellv1(6,5),showtreestrcuture=True,avgmutations=1):
	"""Function that uses microsatelite and coalescent data structures to make a ground truth tree with simulated microsatelites for a cell lineage map """
	#evotree,newickstring=cds.makeemptycoalescentmodel(initnumcells,inverlambda)
	#print "inverlambda in makecoalescentmicrosatelitetree:", inverlambda
	#print help(cds.makeemptycoalescentmodel), cds.makeemptycoalescentmodel(initnumcells,inverlambda)
	evotree=cds.makeemptycoalescentmodel(initnumcells,inverlambda)
	if showtreestrcuture:cds.roughimageofgraphforcellsms(evotree)
	LCA=cds.findlongestnodename(evotree)
	offspringdic=cds.finddescendants(evotree,LCA)
	dond=cds.makedicofnodedata(evotree)
	#initialize the first cell
	firstcell=startingcell
	cds.updatecelldata(dond,LCA,firstcell)
	#print cds.accesscelldata(dond,LCA)
	parentnodestolookat=set([LCA])
	nodeswithinitcelldata=set([LCA])
	while (len(parentnodestolookat)>0):
		nextparentotlookat=parentnodestolookat.pop()
		if (nextparentotlookat in offspringdic): nextnodestoinitcelldata=offspringdic[nextparentotlookat]
		else: continue
		for nodetoinit in nextnodestoinitcelldata:
			cds.updatecelldata(dond,nodetoinit,cds.accesscelldata(dond,nextparentotlookat).copy())
			for iteration in xrange(avgmutations):cds.accesscelldata(dond,nodetoinit).mutate()
			nodeswithinitcelldata.add(nodetoinit)
			parentnodestolookat.add(nodetoinit)
	assert(len(parentnodestolookat)==0)
	assert(len(nodeswithinitcelldata)==len(evotree.nodes()))
	return evotree#,newickstring

#makebalancedbinarycoalescentmicrosatelitetree

def makebalancedbinarycoalescentmicrosatelitetree(initnumcells=cds.initnumcells,inverlambda=cds.inverlambda,startingcell=makeinitcellv1(6,5),showtreestrcuture=True,avgmutations=1):
	"""Function that uses microsatelite and coalescent data structures to make a ground truth tree with simulated microsatelites for a cell lineage map """
	#evotree,newickstring=cds.makeemptycoalescentmodel(initnumcells,inverlambda)
	evotree=cds.makebalancedbinaryemptycoalescentmodel(initnumcells,inverlambda)
	if showtreestrcuture:cds.roughimageofgraphforcellsms(evotree)
	LCA=cds.findlongestnodename(evotree)
	offspringdic=cds.finddescendants(evotree,LCA)
	dond=cds.makedicofnodedata(evotree)
	#initialize the first cell
	firstcell=startingcell
	cds.updatecelldata(dond,LCA,firstcell)
	#print cds.accesscelldata(dond,LCA)
	parentnodestolookat=set([LCA])
	nodeswithinitcelldata=set([LCA])
	while (len(parentnodestolookat)>0):
		nextparentotlookat=parentnodestolookat.pop()
		if (nextparentotlookat in offspringdic): nextnodestoinitcelldata=offspringdic[nextparentotlookat]
		else: continue
		for nodetoinit in nextnodestoinitcelldata:
			cds.updatecelldata(dond,nodetoinit,cds.accesscelldata(dond,nextparentotlookat).copy())
			for iteration in xrange(avgmutations):cds.accesscelldata(dond,nodetoinit).mutate()
			nodeswithinitcelldata.add(nodetoinit)
			parentnodestolookat.add(nodetoinit)
	assert(len(parentnodestolookat)==0)
	assert(len(nodeswithinitcelldata)==len(evotree.nodes()))
	return evotree#,newickstring

def doescoalescenttreehavemultipleidenticalcells(evotree):
	"""function that checks if a coalescent tree model of microsatelite evolution has multiple that share an identical microsatelite pattern"""
	if len(evotree.nodes())==0:return True #deals with empty graph that is initially passed in a loop for function to make coalescent trees with each cell having a unique pattern of MSLs 
	listofMSLtuples=[]
	for node in evotree.nodes(data=True):
		celldata= node[1]["celldata"]
		MSLtuple=[]
		for MS in celldata.MS:MSLtuple.append(MS.MSL)
		MSLtuple=tuple(MSLtuple)
		if MSLtuple in listofMSLtuples: return True
		else: listofMSLtuples.append(MSLtuple)
	return False

def makecoalescentmicrosatelitetreewithonlyuniquecells(initnumcells=cds.initnumcells,inverlambda=cds.inverlambda,startingcell=makeinitcellv1(6,5),avgmutations=1.0):
	"""function that makes new trees until a tree is made with all cells having unique cell patterns"""
	evotree=nx.Graph()
	while(doescoalescenttreehavemultipleidenticalcells(evotree)):
		#evotree,newickstring=makecoalescentmicrosatelitetree(initnumcells,inverlambda,startingcell,False)
		print "try to make new tree with unique cells"                
		evotree=makecoalescentmicrosatelitetree(initnumcells,inverlambda,startingcell,False,avgmutations)
		#cds.roughimageofgraphforcellsms(evotree)
	return evotree#,newickstring

def makecoalescentmicrosatelitetreewithonlyuniquecellsbalancedbinary(initnumcells=cds.initnumcells,inverlambda=cds.inverlambda,startingcell=makeinitcellv1(6,5),avgmutations=1.0):
	"""function that makes new trees until a tree is made with all cells having balanced cell patterns"""
	evotree=nx.Graph()
	while(doescoalescenttreehavemultipleidenticalcells(evotree)):
		#evotree,newickstring=makecoalescentmicrosatelitetree(initnumcells,inverlambda,startingcell,False)
		print "try to make new tree with unique cells"                
		evotree=makebalancedbinarycoalescentmicrosatelitetree(initnumcells,inverlambda,startingcell,False,avgmutations)
		#cds.roughimageofgraphforcellsms(evotree)
	return evotree#,newickstring


def reformatcelldatav1(name,celldata):
	"""function that reformats the name of each node to include name of cell, name of markers and length of each marker"""
	assert(type(name).__name__=="str")
	assert(type(celldata).__name__=="Cell")
	reformatstring=name+"\n"
	reformatstring+="___\n"
	for mspos in xrange(len(celldata.MSnames)):reformatstring+="%s: %d\n"%(celldata.MSnames[mspos],celldata.MS[mspos].MSL)
	return reformatstring
	
def reformatcelldatav2(name,celldata):
	"""function that reformats the name of each node to include name of markers and length of each marker only"""
	assert(type(name).__name__=="str")
	assert(type(celldata).__name__=="Cell")
	reformatstring=""
	for mspos in xrange(len(celldata.MSnames)):reformatstring+="%s: %d\n"%(celldata.MSnames[mspos],celldata.MS[mspos].MSL)
	return reformatstring
        
def makenamereformatedtree(evotree,reformatfunc=reformatcelldatav1,showtreestrcuture=True):
	""" function tht takes basic simulated cell lineage tree and makes a new tree with nodes named according to a reformat function"""
	namechangedict={}
	for oldnode in evotree.nodes(data=True):
		namechangedict[oldnode[0]]=reformatfunc(oldnode[0],oldnode[1]["celldata"])

	reformatednameevotree=nx.Graph()
	for node in evotree.nodes(data=True):
		reformatednameevotree.add_node(namechangedict[node[0]],celldata=node[1]["celldata"])
	for edge in evotree.edges(data=True):
		reformatednameevotree.add_edge(namechangedict[edge[0]],namechangedict[edge[1]],numberofdivisionsbetween=edge[2]["numberofdivisionsbetween"])
		
	if showtreestrcuture:cds.roughimageofgraphforcellsms(reformatednameevotree)
	return reformatednameevotree

def makesimpledotdotfilefromnxgraph(tree,path):
	"""function thatsaves a simple copy of the tree architecture"""
	simpletree=nx.Graph()
	simpletree.add_nodes_from(tree)
	simpletree.add_edges_from(tree.edges())
	#print simpletree.nodes(),simpletree.edges()
	#nx.drawing.nx_agraph.write_dot(simpletree,path)
	write_dot(simpletree,path)


def makeMStable(tree):
	"""function that takes a networkx coalescent model tree and saves it as a 2d list"""
	assert(type(tree).__name__=="Graph")
	for node in tree.nodes(data=True):
		examplenode=node
		break
	firstrow=[]
	firstrow.append("cell/markers")
	for msname in examplenode[1]["celldata"].MSnames:
		firstrow.append(msname)
	MStable=[firstrow]
	for node in tree.nodes(data=True):
		nextrow=[node[0]]
		for ms in node[1]["celldata"].MS:
			nextrow.append(ms.MSL)
		MStable.append(nextrow)
	return MStable
        
#XXXXXXXXXXXXXXXXx

def makemapofdatatocellnames(tree,mapcelltodata=reformatcelldatav2):
	"""function that reuturns a mapping of the microsatelite patterns to the cells that have them in the tree"""
	#rtree=makenamereformatedtree(tree,reformatcelldatav2,False)
	mapdatatocell={}
	for node in tree.nodes(data=True):
		currentnamelist=mapdatatocell.get(mapcelltodata(node[0],node[1]["celldata"]),[])
		mapdatatocell[mapcelltodata(node[0],node[1]["celldata"])]=currentnamelist+[node[0]]
	return mapdatatocell
        

def merge_nodes(G,nodes, new_node, attr_dict=None, **attr):
	"""
	Merges the selected `nodes` of the graph G into one `new_node`,
	meaning that all the edges that pointed to or from one of these
	`nodes` will point to or from the `new_node`.
	attr_dict and **attr are defined as in `G.add_node`.
	originally from : https://gist.github.com/Zulko/7629206
	"""
	assert(new_node not in nodes)
	G.add_node(new_node, attr_dict, **attr) # Add the 'merged' node
	for n1,n2,data in G.edges(data=True):
		# For all edges related to one of the nodes to merge,
		# make an edge going to or coming from the `new gene`.
		if n1 in nodes:
			G.add_edge(new_node,n2,data)
		elif n2 in nodes:
			G.add_edge(n1,new_node,data)
	for n in nodes: # remove the merged nodes
		G.remove_node(n)

def compressredunantcells(dataonlytree,mapdatatocell):
	"""function that takes a dataonlytree and a mapping of the microsatelite patterns to the cells that have them merges cells that share microsatelite patterns"""
	compressedtree=dataonlytree.copy()
	#cds.accesscelldata(dond,nodetoinit)
	for node in compressedtree.nodes(data=True):
		if node[0] in compressedtree.nodes():
			for entry in mapdatatocell:
				if node[0] in mapdatatocell[entry]:
					if (len(mapdatatocell[entry])>1):
						#print mapdatatocell[entry], ",".join(mapdatatocell[entry]),node
						#merge_nodes(compressedtree,mapdatatocell[entry], ",".join(mapdatatocell[entry]), celldata=node[1]["celldata"])
						merge_nodes(compressedtree,mapdatatocell[entry], (sorted(mapdatatocell[entry],reverse=True)[0])+"*", celldata=node[1]["celldata"])                                                                                                #merge_nodes(compressedtree,mapdatatocell[entry], ",".join(mapdatatocell[entry]), celldata=node[1]["celldata"])
	return compressedtree
            
#XXXXXXXXXXXXXXXXXXXXXXXXX        
        
#def rundatagenerationwithtreeswithuniquecellsbalancedbinary(numiter,numcells=6,lam=1.0,startingcells=makeinitcellv1,numMS=6,initMSL=5,prefix="",addnoise=False,fractioncellstoretain=2/3.0,fractionvalueswithnoise=1/10.0,leavesonly=False,avgmutations=1):
#	"""Function that automatically generates numiter trees with numcells cells which each cell having a unique ms pattern"""
#	"""Note numcells corresponds to number of terminal leaves not total number of cells, total number of cells is 2*numcells-1"""
#	timestr=time.asctime(time.localtime());date= timestr.split(" ");date=" ".join(date[:3]+date[4:]);
#	datadic={};datadic["treefolders"]=[];datadic["nameonlytreefilename"]=[];datadic["nameonlytree"]=[];datadic["newicktree"]=[];datadic["newicktreefilename"]=[];datadic["nameanddatatreefilename"]=[];datadic["nameanddatatree"]=[];datadic["dataonlytreefilename"]=[];datadic["dataonlytree"]=[];datadic["excelname"]=[];datadic["excelsheetname"]=[];datadic["datatonamemaping"]=[];
#	topdir=(prefix+"multi_trees_nondup_cell_patterns_%s"+os.sep)%(date);datadic["datadir"]=topdir;
#       datadic["onlyuniquecellsintree"]=[]
#	if topdir[:-1] not in os.listdir("."):os.mkdir(topdir)
#	os.chdir(topdir)
#	workdir=("init_datasets"+os.sep);datadic["workdir"]=topdir+workdir
#	if workdir[:-1] not in os.listdir("."):os.mkdir(workdir)
#	os.chdir(workdir)
#	startingcells=startingcells(numMS,initMSL)
#	readm="README.txt";readmfile=open(readm,"w");datadic["readmename"]=(topdir+workdir+"README.txt");
#	readmfile.write("Data generation began on %s\n"%(timestr));readmfile.write("Make %d trees each with %d cells that each have %d markers\n"%(numiter,(2*numcells-1),numMS));
#	datadic["addednoise"]=addnoise
#	if addnoise: readmfile.write("Noise added to the data in the form of only %d of the at most %d cells with a %f%% chance that a value for a microsatellite has been altered due to noise\n"%(int(round(fractioncellstoretain*(2*numcells-1))),2*numcells-1,fractionvalueswithnoise*100))
#	else:readmfile.write("No Noise added\n")
#	for iteration in xrange(numiter):#iteration=0#start loop##make a new tree each time run:
#		print "generate tree %d"%(iteration)
#		iterfolder=("generated_tree_number_%d"+os.sep)%(iteration)
#		datadic["treefolders"].append((topdir+workdir+iterfolder))
#		if iterfolder[:-1] not in os.listdir("."):os.mkdir(iterfolder)
#		os.chdir(iterfolder)
#		if hasattr(startingcells,"__iter__"):
#			assert(len(startingcells)==numiter)
#			currentstartingcell=startingcells[iteration]
#		else:currentstartingcell=startingcells
#		datadic["onlyuniquecellsintree"].append(True)
#		evotree=makecoalescentmicrosatelitetreewithonlyuniquecellsbalancedbinary(numcells,lam,currentstartingcell,avgmutations);datadic["nameonlytree"].append(evotree);
#		datadic["datatonamemaping"].append(makemapofdatatocellnames(evotree,reformatcelldatav2))                
#		newickstring=cds.findnewickrepresentationofatreewrapper(evotree);datadic["newicktree"].append(newickstring);
#		makesimpledotdotfilefromnxgraph(evotree,"nameonlytree_iter_%d..dot"%(iteration));datadic["nameonlytreefilename"].append((datadic["treefolders"][iteration]+"nameonlytree_iter_%d..dot"%(iteration)));
#		nwkfile=open("nameonlytree_iter_%d.nwk"%(iteration),"w");nwkfile.write(newickstring);nwkfile.close();datadic["newicktreefilename"].append((datadic["treefolders"][iteration]+"nameonlytree_iter_%d.nwk"%(iteration)));
#		nameanddataevotree=makenamereformatedtree(evotree,reformatcelldatav1,False);datadic["nameanddatatree"].append(nameanddataevotree);
#		makesimpledotdotfilefromnxgraph(nameanddataevotree,"nameandatatree_iter_%d..dot"%(iteration));datadic["nameanddatatreefilename"].append((datadic["treefolders"][iteration]+"nameandatatree_iter_%d..dot"%(iteration)));
#		dataonlyevotree=makenamereformatedtree(evotree,reformatcelldatav2,False);datadic["dataonlytree"].append(dataonlyevotree);
#		makesimpledotdotfilefromnxgraph(dataonlyevotree,"dataonlytree_iter_%d..dot"%(iteration));datadic["dataonlytreefilename"].append((datadic["treefolders"][iteration]+"dataonlytree_iter_%d..dot"%(iteration)));
#		treetorecord=makeMStable(evotree)
#		if addnoise:
#			treetorecord=selectsubsetofcell(treetorecord,fractioncellstoretain,leavesonly)
#			addnoisetofractionofvaluesintable(treetorecord,fractionvalueswithnoise,"int",addnoisetoentry)
#		turntable2excelfile(treetorecord,"MSdataset.xlsx","exampleMSdataset");datadic["excelname"].append((datadic["treefolders"][iteration]+"MSdataset.xlsx"));datadic["excelsheetname"].append("exampleMSdataset");
#		if "MSdataset.xlsx" in os.listdir("."): print "excel file was made"
#		else: print "no file made"
#		os.chdir("..")
#		#end loop
#	readmfile.close()
#	saveddatastructfile=open("saveddatastructs.pkl","w");pickle.dump(datadic,saveddatastructfile);saveddatastructfile.close()
#	os.chdir("..");os.chdir("..");
#	return datadic



def rundatagenerationwithtreeswithuniquecells(numiter,numcells=6,lam=1.0,startingcells=makeinitcellv1,numMS=6,initMSL=5,prefix="",addnoise=False,fractioncellstoretain=2/3.0,fractionvalueswithnoise=1/10.0,leavesonly=False,avgmutations=1):
	"""Function that automatically generates numiter trees with numcells cells which each cell having a unique ms pattern"""
	"""Note numcells corresponds to number of terminal leaves not total number of cells, total number of cells is 2*numcells-1"""
	timestr=time.asctime(time.localtime());date= timestr.split();date=" ".join(date[:3]+date[4:]);#.split reomves all white space
	datadic={};datadic["treefolders"]=[];datadic["nameonlytreefilename"]=[];datadic["nameonlytree"]=[];datadic["newicktree"]=[];datadic["newicktreefilename"]=[];datadic["nameanddatatreefilename"]=[];datadic["nameanddatatree"]=[];datadic["dataonlytreefilename"]=[];datadic["dataonlytree"]=[];datadic["excelname"]=[];datadic["excelsheetname"]=[];datadic["datatonamemaping"]=[];
	topdir=(prefix+"multi_trees_nondup_cell_patterns_%s"+os.sep)%(date);datadic["datadir"]=topdir;
        datadic["onlyuniquecellsintree"]=[]
	if topdir[:-1] not in os.listdir("."):os.mkdir(topdir)
	os.chdir(topdir)
	workdir=("init_datasets"+os.sep);datadic["workdir"]=topdir+workdir
	if workdir[:-1] not in os.listdir("."):os.mkdir(workdir)
	os.chdir(workdir)
	startingcells=startingcells(numMS,initMSL)
	readm="README.txt";readmfile=open(readm,"w");datadic["readmename"]=(topdir+workdir+"README.txt");
	readmfile.write("Data generation began on %s\n"%(timestr));readmfile.write("Make %d trees each with %d cells that each have %d markers\n"%(numiter,(2*numcells-1),numMS));
	datadic["addednoise"]=addnoise
	if addnoise: readmfile.write("Noise added to the data in the form of only %d of the at most %d cells with a %f%% chance that a value for a microsatellite has been altered due to noise\n"%(int(round(fractioncellstoretain*(2*numcells-1))),2*numcells-1,fractionvalueswithnoise*100))
	else:readmfile.write("No Noise added\n")
	for iteration in xrange(numiter):#iteration=0#start loop##make a new tree each time run:
		print "generate tree %d"%(iteration)
		iterfolder=("generated_tree_number_%d"+os.sep)%(iteration)
		datadic["treefolders"].append((topdir+workdir+iterfolder))
		if iterfolder[:-1] not in os.listdir("."):os.mkdir(iterfolder)
		os.chdir(iterfolder)
		if hasattr(startingcells,"__iter__"):
			assert(len(startingcells)==numiter)
			currentstartingcell=startingcells[iteration]
		else:currentstartingcell=startingcells
		datadic["onlyuniquecellsintree"].append(True)
		evotree=makecoalescentmicrosatelitetreewithonlyuniquecells(numcells,lam,currentstartingcell,avgmutations);datadic["nameonlytree"].append(evotree);
		datadic["datatonamemaping"].append(makemapofdatatocellnames(evotree,reformatcelldatav2))                
		newickstring=cds.findnewickrepresentationofatreewrapper(evotree);datadic["newicktree"].append(newickstring);
		makesimpledotdotfilefromnxgraph(evotree,"nameonlytree_iter_%d..dot"%(iteration));datadic["nameonlytreefilename"].append((datadic["treefolders"][iteration]+"nameonlytree_iter_%d..dot"%(iteration)));
		nwkfile=open("nameonlytree_iter_%d.nwk"%(iteration),"w");nwkfile.write(newickstring);nwkfile.close();datadic["newicktreefilename"].append((datadic["treefolders"][iteration]+"nameonlytree_iter_%d.nwk"%(iteration)));
		nameanddataevotree=makenamereformatedtree(evotree,reformatcelldatav1,False);datadic["nameanddatatree"].append(nameanddataevotree);
		makesimpledotdotfilefromnxgraph(nameanddataevotree,"nameandatatree_iter_%d..dot"%(iteration));datadic["nameanddatatreefilename"].append((datadic["treefolders"][iteration]+"nameandatatree_iter_%d..dot"%(iteration)));
		dataonlyevotree=makenamereformatedtree(evotree,reformatcelldatav2,False);datadic["dataonlytree"].append(dataonlyevotree);
		makesimpledotdotfilefromnxgraph(dataonlyevotree,"dataonlytree_iter_%d..dot"%(iteration));datadic["dataonlytreefilename"].append((datadic["treefolders"][iteration]+"dataonlytree_iter_%d..dot"%(iteration)));
		treetorecord=makeMStable(evotree)
		treetorecord=selectsubsetofcell(treetorecord,fractioncellstoretain,leavesonly)
		if addnoise:
			addnoisetofractionofvaluesintable(treetorecord,fractionvalueswithnoise,"int",addnoisetoentry)
		turntable2excelfile(treetorecord,"MSdataset.xlsx","exampleMSdataset");datadic["excelname"].append((datadic["treefolders"][iteration]+"MSdataset.xlsx"));datadic["excelsheetname"].append("exampleMSdataset");
		if "MSdataset.xlsx" in os.listdir("."): print "excel file was made"
		else: print "no file made"
		os.chdir("..")
		#end loop
	readmfile.close()
	saveddatastructfile=open("saveddatastructs.pkl","w");pickle.dump(datadic,saveddatastructfile);saveddatastructfile.close()
	os.chdir("..");os.chdir("..");
	return datadic

#cds.findlongestnodename(nodenamelist)

def selectsubsetofcell(table,fractionentriestoretain,leavesonly):
    print "number all cells to keep, number in original table, fraction to retain:", int(round(len(table[1:])*fractionentriestoretain)),len(table),fractionentriestoretain
    if leavesonly:
            newtable=[table[0]]
            for row in table[1:]:
                    if "|" not in row[0]:
                          newtable.append(row)
            return ([newtable[0]]+rand.sample(newtable[1:],int(round(len(newtable[1:])*fractionentriestoretain))))
    else:
            return ([table[0]]+rand.sample(table[1:],int(round(len(table[1:])*fractionentriestoretain))))

def addnoisetoentry(value):
    return value+rand.choice([-1,1])

def addnoisetofractionofvaluesintable(table,fractionentrieswithnoise,entrytype,noisefunc):
    for rowpos in xrange(len(table)):
        for colpos in xrange(len(table[rowpos])):
            if (type(table[rowpos][colpos]).__name__==entrytype):
                if (rand.random()<fractionentrieswithnoise): table[rowpos][colpos]=noisefunc(table[rowpos][colpos])


#rundatagenerationwithtrees(1,numcells=6,lam=1.0,startingcells=makeinitcellv1,numMS=6,initMSL=5,prefix="",addnoise=True,fractioncellstoretain=2/3.0,fractionvalueswithnoise=1/10.0)
##def rundatagenerationwithtrees(numiter,numcells=6,lam=1.0,startingcells=makeinitcellv1,numMS=6,initMSL=5,prefix="",addnoise=False,fractioncellstoretain=2/3.0,fractionvalueswithnoise=1/10.0,leavesonly=False,avgmutations=1):
##	"""Function that automatically generates numiter trees with numcells cells"""
##	"""Note numcells corresponds to number of terminal leaves not total number of cells, total number of cells is 2*numcells-1 at most"""
##	timestr=time.asctime(time.localtime());date= timestr.split(" ");date=" ".join(date[:3]+date[4:]);
##	datadic={};datadic["treefolders"]=[];datadic["nameonlytreefilename"]=[];datadic["nameonlytree"]=[];datadic["newicktree"]=[];datadic["newicktreefilename"]=[];datadic["nameanddatatreefilename"]=[];datadic["nameanddatatree"]=[];datadic["dataonlytreefilename"]=[];datadic["dataonlytree"]=[];datadic["excelname"]=[];datadic["excelsheetname"]=[];datadic["datatonamemaping"]=[];datadic["onlyuniquecellsintree"]=[];
##	datadic["compressedtreedata"]={};datadic["compressedtreedata"]["nameonlytreefilename"]=[];datadic["compressedtreedata"]["nameonlytree"]=[];datadic["compressedtreedata"]["newicktree"]=[];datadic["compressedtreedata"]["newicktreefilename"]=[];datadic["compressedtreedata"]["nameanddatatreefilename"]=[];datadic["compressedtreedata"]["nameanddatatree"]=[];datadic["compressedtreedata"]["dataonlytreefilename"]=[];datadic["compressedtreedata"]["dataonlytree"]=[];datadic["compressedtreedata"]["datatonamemaping"]=[];
##	topdir=(prefix+"multi_trees_nondup_cell_patterns_%s_withnoiseadded_%s"+os.sep)%(date,addnoise);datadic["datadir"]=topdir;
##	if topdir[:-1] not in os.listdir("."):os.mkdir(topdir)
##	os.chdir(topdir)
##	workdir=("init_datasets"+os.sep);datadic["workdir"]=topdir+workdir
##	if workdir[:-1] not in os.listdir("."):os.mkdir(workdir)
##	os.chdir(workdir)
##	startingcells=startingcells(numMS,initMSL)
##	readm="README.txt";readmfile=open(readm,"w");datadic["readmename"]=(topdir+workdir+"README.txt");
##	readmfile.write("Data generation began on %s\n"%(timestr));readmfile.write("Make %d trees each with %d cells that each have %d markers\n"%(numiter,(2*numcells-1),numMS));
##	datadic["addednoise"]=addnoise
##	if addnoise: readmfile.write("Noise added to the data in the form of only %d of the at most %d cells with a %f%% chance that a value for a microsatellite has been altered due to noise\n"%(int(round(fractioncellstoretain*(2*numcells-1))),2*numcells-1,fractionvalueswithnoise*100))
##	else:readmfile.write("No Noise added\n")
##	for iteration in xrange(numiter):#iteration=0#start loop##make a new tree each time run:
##		print "generate tree %d"%(iteration)
##		iterfolder=("generated_tree_number_%d"+os.sep)%(iteration)
##		datadic["treefolders"].append((topdir+workdir+iterfolder))
##		if iterfolder[:-1] not in os.listdir("."):os.mkdir(iterfolder)
##		os.chdir(iterfolder)
##		if hasattr(startingcells,"__iter__"):
##			assert(len(startingcells)==numiter)
##			currentstartingcell=startingcells[iteration]
##		else:currentstartingcell=startingcells
##		#inittree
##		evotree=makecoalescentmicrosatelitetree(numcells,lam,currentstartingcell,False,avgmutations);datadic["nameonlytree"].append(evotree);
##		datadic["datatonamemaping"].append(makemapofdatatocellnames(evotree,reformatcelldatav2))                
##		newickstring=cds.findnewickrepresentationofatreewrapper(evotree);datadic["newicktree"].append(newickstring);
##		makesimpledotdotfilefromnxgraph(evotree,"nameonlytree_iter_%d..dot"%(iteration));datadic["nameonlytreefilename"].append((datadic["treefolders"][iteration]+"nameonlytree_iter_%d..dot"%(iteration)));
##		nwkfile=open("nameonlytree_iter_%d.nwk"%(iteration),"w");nwkfile.write(newickstring);nwkfile.close();datadic["newicktreefilename"].append((datadic["treefolders"][iteration]+"nameonlytree_iter_%d.nwk"%(iteration)));
##		nameanddataevotree=makenamereformatedtree(evotree,reformatcelldatav1,False);datadic["nameanddatatree"].append(nameanddataevotree);
##		makesimpledotdotfilefromnxgraph(nameanddataevotree,"nameandatatree_iter_%d..dot"%(iteration));datadic["nameanddatatreefilename"].append((datadic["treefolders"][iteration]+"nameandatatree_iter_%d..dot"%(iteration)));
##		dataonlyevotree=makenamereformatedtree(evotree,reformatcelldatav2,False);datadic["dataonlytree"].append(dataonlyevotree);
##		makesimpledotdotfilefromnxgraph(dataonlyevotree,"dataonlytree_iter_%d..dot"%(iteration));datadic["dataonlytreefilename"].append((datadic["treefolders"][iteration]+"dataonlytree_iter_%d..dot"%(iteration)));
##		#make tree where nodes with no identical cells
##		if not doescoalescenttreehavemultipleidenticalcells(evotree):
##			datadic["onlyuniquecellsintree"].append(True)
##			datadic["compressedtreedata"]["nameonlytreefilename"].append(None);datadic["compressedtreedata"]["nameonlytree"].append(None);datadic["compressedtreedata"]["newicktree"].append(None);datadic["compressedtreedata"]["newicktreefilename"].append(None);datadic["compressedtreedata"]["nameanddatatreefilename"].append(None);datadic["compressedtreedata"]["nameanddatatree"].append(None);datadic["compressedtreedata"]["dataonlytreefilename"].append(None);datadic["compressedtreedata"]["dataonlytree"].append(None);datadic["compressedtreedata"]["datatonamemaping"].append(None);
##			#finally make excel
##			treetorecord=makeMStable(evotree)
##			if addnoise:
##				treetorecord=selectsubsetofcell(treetorecord,fractioncellstoretain,leavesonly)
##				addnoisetofractionofvaluesintable(treetorecord,fractionvalueswithnoise,"int",addnoisetoentry)
##			turntable2excelfile(treetorecord,"MSdataset.xlsx","exampleMSdataset");datadic["excelname"].append((datadic["treefolders"][iteration]+"MSdataset.xlsx"));datadic["excelsheetname"].append("exampleMSdataset");
##			if "MSdataset.xlsx" in os.listdir("."): print "excel file was made"
##			else: print "no file made"
##
##		else: 
##			#read
##			datadic["onlyuniquecellsintree"].append(False)
##			mapofdatatocell=makemapofdatatocellnames(evotree,reformatcelldatav2);
##			uniquecelltree=compressredunantcells(evotree,mapofdatatocell);datadic["compressedtreedata"]["nameonlytree"].append(uniquecelltree);
##			datadic["compressedtreedata"]["datatonamemaping"].append(makemapofdatatocellnames(uniquecelltree,reformatcelldatav2))                
##			if nx.is_tree(uniquecelltree):newickstring=cds.findnewickrepresentationofatreewrapper(uniquecelltree);datadic["compressedtreedata"]["newicktree"].append(newickstring);nwkfile=open("compressed_nameonlytree_iter_%d.nwk"%(iteration),"w");nwkfile.write(newickstring);nwkfile.close();datadic["compressedtreedata"]["newicktreefilename"].append((datadic["treefolders"][iteration]+"compressed_nameonlytree_iter_%d.nwk"%(iteration)));
##			else:newickstring=None;datadic["compressedtreedata"]["newicktree"].append(None);datadic["compressedtreedata"]["newicktree"].append(newickstring);nwkfile=open("compressed_nameonlytree_iter_%d.nwk"%(iteration),"w");print newickstring;nwkfile.write(str(newickstring));nwkfile.close();datadic["compressedtreedata"]["newicktreefilename"].append((datadic["treefolders"][iteration]+"compressed_nameonlytree_iter_%d.nwk"%(iteration)));
##			makesimpledotdotfilefromnxgraph(uniquecelltree,"compressed_nameonlytree_iter_%d..dot"%(iteration));datadic["compressedtreedata"]["nameonlytreefilename"].append((datadic["treefolders"][iteration]+"compressed_nameonlytree_iter_%d..dot"%(iteration)));
##			nameanddataevotree=makenamereformatedtree(uniquecelltree,reformatcelldatav1,False);datadic["compressedtreedata"]["nameanddatatree"].append(nameanddataevotree)     
##			makesimpledotdotfilefromnxgraph(nameanddataevotree,"compressed_nameanddatatree_iter_%d..dot"%(iteration));datadic["compressedtreedata"]["nameanddatatreefilename"].append((datadic["treefolders"][iteration]+"compressed_nameanddatatree_iter_%d..dot"%(iteration)));
##			dataonlyevotree=makenamereformatedtree(uniquecelltree,reformatcelldatav2,False);datadic["compressedtreedata"]["dataonlytree"].append(dataonlyevotree);
##			makesimpledotdotfilefromnxgraph(dataonlyevotree,"compressed_dataonlytree_iter_%d..dot"%(iteration));datadic["compressedtreedata"]["dataonlytreefilename"].append((datadic["treefolders"][iteration]+"dataonlytree_iter_%d..dot"%(iteration)));
##			#finally make excel 
##			treetorecord=makeMStable(uniquecelltree)
##			if addnoise:
##				treetorecord=selectsubsetofcell(treetorecord,fractioncellstoretain,leavesonly)
##				addnoisetofractionofvaluesintable(treetorecord,fractionvalueswithnoise,"int",addnoisetoentry)
##			turntable2excelfile(treetorecord,"MSdataset.xlsx","exampleMSdataset");datadic["excelname"].append((datadic["treefolders"][iteration]+"MSdataset.xlsx"));datadic["excelsheetname"].append("exampleMSdataset");
##			if "MSdataset.xlsx" in os.listdir("."): print "excel file was made"
##			else: print "no file made"
##		os.chdir("..")
##		#end loop
##	readmfile.close()
##	saveddatastructfile=open("saveddatastructs.pkl","w");pickle.dump(datadic,saveddatastructfile);saveddatastructfile.close()
##	os.chdir("..");os.chdir("..");
##	return datadic
