#!/usr/bin/python

import networkx as nx
import random as rand
import numpy as np


def addone(MSL):
	"""basic dummy function to mutate a MSL length"""
	return [MSL+1]
    
def simplestepwise(MSL):
	"""SMM,basic MSL mutation model, simpliest, that increases or decreases, at random, the length of a MSL """
	if(MSL==0):newMSL=0
	else:
		#print (rand.sample([-1,1],1))[0], MSL
		newMSL=MSL+(rand.sample([-1,1],1))[0]
	return newMSL

def simplestepwisewithuppercutoff(MSL,upperlimit=10**10):
	"""improved MSL mutation model, that increases or decreases, at random, the length of a MSL, with the likelihood of increase or decreasing changing linearly depending on how close the MSL is to the upperlimit """
	percentlikelytodecrease=float(MSL)/upperlimit#old-percentlikelytodecrease=int(MSL*100.0/upperlimit)
	#print len([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)), ([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)).count(-1), ([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)).count(1)
	if(MSL==0):newMSL=0
	#else:newMSL=MSL+(rand.sample(([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)),1))[0]
	else:
		if (rand.random()<percentlikelytodecrease):newMSL=MSL-1
		else: newMSL=MSL+1
	return [newMSL]
	
def simplestepwisewithuppercutoffold(MSL,upperlimit=10**10): #make for only multiple
	"""improved MSL mutation model, that increases or decreases, at random, the length of a MSL, with the likelihood of increase or decreasing changing linearly depending on how close the MSL is to the upperlimit """
	percentlikelytodecrease=float(MSL)/upperlimit#old-percentlikelytodecrease=int(MSL*100.0/upperlimit)
	#print len([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)), ([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)).count(-1), ([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)).count(1)
	if(MSL==0):newMSL=0
	#else:newMSL=MSL+(rand.sample(([-1]*percentlikelytodecrease+[1]*(100-percentlikelytodecrease)),1))[0]
	else:
		if (rand.random()<percentlikelytodecrease):newMSL=MSL-1
		else: newMSL=MSL+1
	return [newMSL]
	
def simpleslippageandpointmutation(MSL,lldp=.01,slippagelikelihoodprobability=.957,mode=None):
	"""improved MSL mutation model, that incorporates the possibility of MSL slippage, point mutation to break MSL length and a transition from MSL of 1 to 2 """
	"from paper"
	"""def mrun(num):
	lst=[]
	for i in xrange(num):
		print m.MSL,
		a=m.mutate(mode="lld")
		print a
		lst.append(a[0])
		lst.append(a[1])
	return lst"""
	assert(type(MSL).__name__=="int")
	slippagemodelmutation=simplestepwise
	if mode=="zero":MSL=0
	if mode=="lld":MSL=1
	if mode=="slip":slippagelikelihoodprobability=10
	if mode=="split":slippagelikelihoodprobability=0
	if (MSL==0):
		mode="stuck at zero"
		#print mode
		newMSL=0
	elif (MSL==1): 
		mode="low likelihood duplication possible"
		#print mode
		newMSL=lowlikelihoodduplication(MSL,lldp)
	else:
		if (rand.random()<slippagelikelihoodprobability):
			mode="slippage model used"
			#print mode
			newMSL=slippagemodelmutation(MSL)# default value of .957 for slippagelikelihoodprobability correpsonds to a=1 × 10−8, b=2.2 × 10−6 , b/a=22, b/(b+a)=.957
		else:
			mode="splitting by point mutation"
			#print mode
			newMSL=pointmutationsplittingmutation(MSL)# parameter a/(b/a) =1-slippagelikelihoodprobability
	return [newMSL,mode]
	
#######subfunctions for simpleslippageandpointmutation##################
def lowlikelihoodduplication(MSL,lldp):
	if (rand.random()<lldp):return 2*MSL
	else:return MSL
	
def pointmutationsplittingmutation(MSL):
	return MSL - rand.randint(1,MSL-1)#MSL length decreases from MSL to anywhere between 1 and MSL-1 (equivalent to MSL-[1,MSL-1])
	

################################################################## 


class Microsatelite(object):

	def init_checks(self):
		assert(type(self.MSL).__name__=="int")
		assert(self.MSL>=0)
		if "uplim" in self.param:assert(type(self.upperlimit).__name__=="int")
		if self.mutmodel.__name__=="simplestepwisewithuppercutoff":assert("uplim" in self.param)
		if self.mutmodel.__name__=="simpleslippageandpointmutation":assert(("lldp-slp" in self.param))
		print "no problem with init"
	
	def __init__(self,MSL,mutationmodel,requiredparam="",upperlimit="doubleMSL",lldp=.01,slippagelikelihoodprobability=.957):
		#MSL is the Microsatelite length
		assert(type(MSL).__name__=="int")
		self.MSL=MSL
		self.mutmodel=mutationmodel
		if (upperlimit=="doubleMSL"):self.upperlimit=2*self.MSL
		else:self.upperlimit=upperlimit
		self.lldp=lldp
		self.slippagelikelihoodprobability=slippagelikelihoodprobability
		#param are "uplim","lldp-slp"
		self.param=requiredparam
		self.init_checks()
		"""m=Microsatelite(5,addone)"""
	
	def __str__(self):
		return "A Microsatelite with a length of %d that use a mutation model with the name of %s"%(self.MSL,self.mutmodel.__name__)
	
	def copy(self):
		return Microsatelite(self.MSL,self.mutmodel,self.param,self.upperlimit,self.lldp,self.slippagelikelihoodprobability)
	
	def mutate(self,mode=None):
		#make tuple for input
		inputlist=[self.MSL]
		#print self.param, tuple(inputlist)
		if ("uplim" in self.param):inputlist=inputlist+[self.upperlimit]
		#else:self.MSL=self.mutmodel(self.MSL)
		if ("lldp-slp" in self.param):inputlist=inputlist+[self.lldp,self.slippagelikelihoodprobability]
		#print self.param, tuple(inputlist)
		#print tuple(inputlist)
		if mode!=None:inputlist=inputlist+[mode]
		data=self.mutmodel(*tuple(inputlist))
		#print data
		self.MSL=data[0]
		#print self.MSL
		return data
	
		
